/******************************************************************************/
//
// Name: 	Interfacing ADSP-21262 with MMC (MultiMedia Card) on SPI
//
/******************************************************************************

(C) Copyright 2005 - Analog Devices, Inc.  All rights reserved.

File Name:		MMC_Interface.c

Date Modified:	01/02/05	-	JR	-	Rev 1.0

Software:       VisualDSP++3.5 (November update)

Purpose:		To test basic MMC commands in SPI mode
				
*******************************************************************************/

// Include the mmc header file
#include "mmc.h"

// Pointers to source and destination buffers
int *source;
int *dest;

void main()
{
	int i;
	
	// Pin9 used as a flag to select MMC chip select
	// Initially CS is made logic "HI".
	SRU(HIGH, DAI_PB09_I);
	SRU(HIGH, PBEN09_I);

	// Pin10 is used to trigger the oscilloscope
	SRU(LOW, DAI_PB10_I);
	SRU(HIGH, PBEN10_I);
	
	// Initialize MMC Card 
	// Sends CMD0 and CMD1 to the MMC Card
	mmc_init();
	
	// As initialization is done, baud rate could be increased.
	*pSPIBAUD = 300; // BAUDR = 150 => SPI Baud Rate = 200 / (4*150) = 333.33 KHz

	// Set the block length - CMD16
	mmc_set_block_length(512);
	
	// Allocate memory for the source buffer
	// Data block write size is fixed to 512 in MMC cards
	// Allocating 515 to accomodate data token and crc bytes
	// First byte: Data token
	// Last two bytes: CRC
	source = (int *)malloc(515 * sizeof(int));
	
	// Initialize source data 
	// Write block length is fixed to 512
	// First word is data token and last two words are CRC bytes
	for(i=0;i<515;i++)
	{
		// First value should be 0xFE - DATA Token
		if (i==0)
		{
			*source = 0xFE;
		}
		// Last two values should be 0xFF
		else if ((i==513)||(i==514))
		{
			*(source+i) = 0xFF;
		}
		else
			*(source+i) = i & 0xFF; // User specific data
	}
	
	// Write Data to MMC - CMD24
	// Write block length is fixed to 512
	// Data written is stored in the location pointed by "source"
	mmc_write_data(0x4000,source);
	
	// Allocate memory for destination data
	// Data allocated should be block transfer block length + 2 
	// Additional 2 words are for accomdating two CRC bytes.
	dest = (int *)malloc(514 * sizeof(int));
	
	// Read Data from MMC - CMD17
	// Start reading 512 bytes from address 0x4000 into
	// the location pointed by "dest"
	mmc_read_data(0x4000,512,dest);

	// Data comparison
	for(i=0;i<512;i++)
	{
		if(*(source+i+1)!=*(dest+i))
		{
			printf("Data not matching at location %d, source: 0x%x dest: 0x%x\n",i,*(source+i+1),*(dest+i));
			exit(1);
		}
	}
	
	printf("Data is written correctly to MMC card\n");
}
